/* 
   -----------------------------------------------------------------------
   Program.....: internetupdate.prg

   Programmer..: Bernd Hohenester  (newsletter@hobe.de)

   Date........: November 2006 for dB-D-A-CH conference

   Notes.......: a program for updating an application over the internet
   
                 This is only one way to make an internet update. Please feel free
                 to use and change this program as you need.

   Dependencies: client.cc            - class for granting exclusive access
                 ini.cc               - wrapper for ini-files
                 urld.cc              - class for downloading files from internet
                 progresscallback.dll - shows progressbar while download. You need to deploy this file with your app.
                 registry.prg         - wrapper for reading from and writing to registry

   ---------------------------------------------------------------------
*/

/*

1st step: download file with update infos

The infos are held in an ini-file, because you can store many infos in a 
structured way and there are ready routines to access the infos.

*/

// you really need exclusive access to the application
// one way to accomplish this is a simple user manager in client.cc
// you have to start it in the init-routine of your application
// "dBDACH" is a database-name; change it to your needs

set procedure to client.cc additive
_app.cl = new client()
if not _app.cl.login("dBDACH")
  return
endif


// switch to exclusive access
if not _app.cl.exclusive()
  return
endif


// define the default values for the host and the filename
// change this to your needs
#define INTERNET_HOST   "http://www.hobe.de/download/"
#define INTERNET_FILE   "dbdach.ini"


local cHost, cFile, cLocalFile, f


set procedure to ini.cc additive
// _app.ini = new ini(<INI-Name>)


// read server name from application ini file
cHost = INTERNET_HOST
// _app.ini.Default := INTERNET_HOST
// cHost = _app.ini.getValue("Update", "Server")

// add backslash
cHost += iif(cHost.right(1) <> "/", "/", "")


// read ini filename
cFile = INTERNET_FILE
// _app.ini.Default := INTERNET_FILE
// cFile = _app.ini.getValue("Update", "File")


// local temporary file
cLocalFile = set("directory") + "\~temp.ini"


// now download the ini file with the update infos from internet
set procedure to urld.cc additive
f = new dwnldcnt(cHost + cFile, cLocalFile)
f.readmodal()
f.release()


// now dbdach.ini from internet was saved as local file ~temp.ini
// if not - cancel the program
if not new file().exists(cLocalFile)
  msgbox(cHost + cFile + " does not exist!","Internetupdate",48)
  _app.cl.unlock()
  return
endif




/*

2nd step: read the update infos

*/


// define default values for application and version number
// change this to your needs
#define APP_NAME     "Mitgliederverwaltung dB-D-A-CH "
#define APP_VERSION  "1.0"
  
  
local lRunUpdate, oIni, cInternetShortFileName, cInternetLongFileName, dInternetBuild

lRunUpdate = false


// read the values from ~temp.ini
oIni = new ini(cLocalFile)

// LongFileName is an additional option, if the update-file is in another webspace
cInternetLongFileName = oIni.getvalue(APP_NAME.righttrim(), "LongFileName")
  
// Build is the date from the update-file
dInternetBuild = ctod(oIni.getvalue(APP_NAME.righttrim(), "Build"))
  
// ShortFileName is the name of the update-file and a switch:
// if not set, there is actually no update
cInternetShortFileName = oIni.getvalue(APP_NAME.righttrim(), "ShortFileName")

oIni.release()
oIni = null
close procedure ini.cc



if not empty(cInternetShortFileName)

  // now read the values from the installed application from registry
  private dBuildLocal
  local oReg
  
  #include winreg.h
  set procedure to registry.prg additive
  
  // you need to change this to your needs
  oReg = new registry(HKEY_LOCAL_MACHINE, ;
    "Software\Bernd Hohenester\" + APP_NAME.righttrim() + "\" + APP_VERSION, false)
  
  
  // dBuildLocal is the date of the installed application
  dBuildLocal = oReg.queryValue("Build")
  if type("dBuildLocal") == "L"  // false, d. h. noch unbelegt
    dBuildLocal = "01.01.1980"
  endif
  dBuildLocal = ctod(dBuildLocal)
  
  oReg.close()
  oReg = null
  close procedure registry.prg
  
  
  
  
  /*
  
  3rd step: compare the dates
  
  */
  
  
  // is the update newer?
  if dBuildLocal < dInternetBuild
    
    local cMsg
    
    cMsg = "There is an update:" + chr(13)
    cMsg += APP_NAME.righttrim() + chr(13)
    cMsg += "Do you want to start the download?"
    
    // ask user
    if (msgbox(cMsg,"Internetupdate",4 + 48) == 6)
      
      
      
      
      /*
      
      4th step: download the update-file
      
      */
      
      local cOldPath, lDirExists
      private cPath
      
      // does folder for the update-file exist?
      if NOT "RUNTIME" $ upper( version(1) )
        cPath = set("DIRECTORY") + "\Update"
      else
        cPath = _dbwinhome + "Update"
      endif
      
      // store current directory
      cOldPath = set("DIRECTORY")
      
      // check wether folder exists
      lDirExists = true
      
      try
        cd (cPath)
      catch(exception e)
        lDirExists = false
      endtry
      
      // restore current directory
      cd (cOldPath)
      
      // make update folder
      if not lDirExists
        try
          md &cPath.
        catch( Exception e )
        endtry
      endif
      
      cPath += iif(cPath.right(1) <> "\", "\", "")
      
      
      
      local cSource, cDest, f, cBatFile, i
      
      // download update file
      cSource = cHost + cInternetShortFileName
      if cInternetLongFileName.length > 0
        cSource = cInternetLongFileName
      endif
      cDest = cPath + cInternetShortFileName
      
      f = new dwnldcnt(cSource, cDest)
      f.readmodal()
      
      
      
      
      /*
      
      5th step: start installation routine
      
      */
      
      if f.errorcode == 0
        lRunUpdate = true
        run(true, cDest)
      endif
      
      f.release()
      
    endif
  else
    msgbox("The application is actual","Internetupdate",48)
  endif
endif




/*

6th step: quit application

*/

// clean up
close procedure urld.cc

// delete temporary file
new file().delete(cLocalFile)

// leave exclusive mode
_app.cl.unlock()


// logout user
_app.cl.logout()
release object _app.cl
_app.cl = null


if lRunUpdate
  // quit application
  quit
endif

return
