/* 
   File:    Class API_Structs.cc
	Autor:   Marc Van den Berghen

   Version:	1.23

   Provide some ease of use for different API-Stuctures
   The Dot-Notation can be used for the Members of the struct
   In order to follow as close as possible the definition of the API,
   the methods of the different classes are named as the members of 
   the structs.

   One purpose of this class is to be able to use pointer in structures
   (see methods settext and gettext)

   Example  this.lv_column1=new lv_column()
   			this.lv_column1.cx(100)

            this sets the cx-Member of the lv_column structure to 100 
            pixels without the parameter (100) , the value of cx 
            is retrieved.


	*************************************************************************
   * This File can be used by everyone! No permission of the author needed!*
   *																								*
   *	BUT PLEASE :  If you add structures to the file, or add methods to   *
   * ===========		the base class (word, byte) send me a copy at			*
   *																								*
   *										info@vdblogic.de									*
   *   Thanks																					*
   *************************************************************************

   History : Version 1.0 		22-Jul-03		Marc Van den Berghen		
                              rudimental Base-Class ,LV_COLUMN, found a way 
                              to include String-pointer in a structure
                              25-Jul-03		Marc VdB
                              LV_ITEM,DRAWITEMSTRUCT
                              29-07-03			Marc VdB
										MEASUREITEMSTRUCT,	
                              01-09-03			Marc VdB
                              ProgressCallbackStruct (downloadprogressbar)
             Version 1.1		06-Sep-03 		Marc VdB
             						Cleaner implementation of the settext-method 	
             Version 1.2		14-09-03    Marc VdB
             						Added stringhandling to the base-class (Ansi only)
                              wordhandling 
             Version 1.21		FLDDesc,CRTblDesc,CURProps,StringPointer
             						Handlepointer for table-restructuring
				 Version 1.22		Bugfix, handling the same structure more then once	
             Version 1.23		added necessary members for table-level change

             to do in next version :  make groups of structures
             					  f.i   listview-related, bde-related , gdi ...	
*/                            


**************************
* Base for all structures
**************************

#define GMEM_FIXED 0

Class Struct_Base(nSize) Of String
	if empty(nSize) 
   	nSize=128
   endif
   nSize = iif((nSize % 2)=1,nSize+1,nSize) //Enlarge to WordBorder for Unicode
	this.String = Replicate(Chr(0),nSize / 2) // Initialize with 0
	this.SizeOf = nSize                       // size of struct
   
	extern chandle 	GlobalAlloc(CUINT, clong) kernel32
   extern chandle    GlobalFree(chandle) kernel32
   extern chandle    GetCurrentProcess( cvoid ) kernel32
   extern clogical   WriteProcessMemory(chandle, cptr, cstring, clong, ;
                     cptr ) kernel32
   extern clogical   WriteProcessMemoryP(chandle, cptr, cptr, clong, ;
                     cptr ) kernel32 from 'WriteProcessMemory'

   function getLong(nIndex)			// get the long(dword) value 
     local n
     n = this.getByte(nIndex) + ;
         bitlshift(this.getByte(nIndex + 1), 8) + ;
         bitlshift(this.getByte(nIndex + 2), 16) + ;
         bitlshift(this.getByte(nIndex + 3), 24)
   return n	

   function setLong(nIndex, nValue)		// set the long value at the desired 
     this.setByte(nIndex, bitand(nValue, 0xFF))    // position in the structure
     this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
     this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
     this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))
   return                    

   function release()		// cleanup
     this.string = ''
     release object this
   return

   /* 
     the following method is used to overload the member-functions 
     of the subclassed structures, so that the same name can be used 
     to set or to get the values (by checking the number of parameters)
   */

   function member(nIndex,nValue)
      if pcount() = 1						
         return this.getlong(nIndex)
      else
         this.setlong(nIndex, nValue)
      endif
   return nValue 

   /* 
      Windows often uses pointers to strings in structures. The 
      following method was specially designed to enable the 
      dbase developper to use such structures. It allocates memory 
      to hold the string and gets the address of this memory
      block from the OS.
   */
   function settext
      parameters c,n,h		// pass pointer as reference so it 
                           // can be changed                  
      local apierror,cResult,hprocess
      cResult =""
      if type("c")="U"						
         return cresult			  			
      else
         if h#0				// release the old memory-block so that 
                           // it can be reused later (and to 
                           // avoid memory-leaks)
            result=globalfree(h)
         endif
         if type("c")="C" 
            h=globalalloc(GMEM_FIXED,len(c)+1)// get the address
            if h=0
               Msgbox("Memory allocation failed","Fatal Error",0)
            else
               hprocess=getcurrentprocess()
               // write the string to the reserved memory-location
               apierror=writeprocessmemory(hprocess,h,c+chr(0), ;
                                          len(c)+1,0)
               if not apierror
                  Msgbox("Pointer creation failed","Fatal Error",0)
               else 
                  cresult=str(h)
                  this.setlong(n,h)
               endif
            endif
         else
            this.setlong(n,c)
         endif
      endif
   return cresult	 // either empty string (on error) or 
                   // pointer to the string

   /*
     !!!!!!!!!!  This method is not fully implemented yet !!!!!!!
     (the value of the string at the position pointed to is not 
     yet returned)

   */
   function gettext(n,h)
      local cresult
      cresult=""
      h=this.getlong(n)
   return

   /* 
      The following function is used to get and set string-values 
      in a structure 

      !!! Attention : it handles only ANSI-Characters (singlebyte)
      Unicode not yet implemented

      nmax : must contain the maximum space accorded to the specific 
            string-member (used to avoid overwriting of the 
            other members)
   */
   function AnsiString(nIndex,nmax,cValue)
      if pcount() = 2						
         return this.getansistring(nIndex,nmax)
      else
         this.setansistring(nIndex, nmax, cvalue)
      endif
   return cValue 

   function SetAnsiString(nIndex,nmax,cValue)
      local i
      nmax=iif(len(cvalue)<nmax-1,len(cvalue)-1,nmax-2)
      for i= 0 to nmax
         this.setbyte(nIndex+i,asc(cValue.charat(i)))
      endfor
   return

   function getansistring(nIndex,nmax)     // assuming null-termination
      local cValue,zw
      cvalue=''
      for i=0 to nmax-1
         zw=this.getbyte(nindex+i)
         if zw # 0
            cvalue+=chr(zw)
         else
            i=nmax
         endif
      endfor
   return cValue 

   /* 
      get and set two-byte values (words)
   */
   function wordmember(nIndex,nValue)
      if pcount() = 1						
         return this.getword(nIndex)
      else
         this.setword(nIndex, nValue)
      endif
   return nValue 

   function getWord(nIndex)			// get the dword value 
     local n
     n = this.getByte(nIndex) + ;
         bitlshift(this.getByte(nIndex + 1), 8) 
   return n	

   function setWord(nIndex, nValue)		// set the word value at the desired 
     this.setByte(nIndex, bitand(nValue, 0xFF))    // position in the structure
     this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
   return 

   function fillstruct(lp_str)
      local hprocess,apierror
      hprocess=getcurrentprocess()
      apierror=writeprocessmemoryp(hprocess,this.string,lp_str,this.sizeof,0)
      if not apierror
         Msgbox("Pointer not valid","Fatal Error",0)
      endif
   return 

endclass

*****************************
* End of Base-Class
*****************************

*********************
* LV_COLUMN structure
*********************
/*
   The LV_COLUMN structure contains information about a column in a 
   list view control. This structure is also used to receive 
   information about a column.

   typedef struct _LV_COLUMN {  
       UINT mask; 
       int fmt; 
       int cx; 
       LPTSTR pszText; 
       int cchTextMax; 
       int iSubItem; 
   } LV_COLUMN; 
*/ 

class LV_COLUMN of Struct_Base(24)

	this.p_psztext=0		// pointer to psztext

   * specifies which member of the struct contains valid data
	function mask(n)							
	return iif(pcount()=0,this.member(0),this.member(0,n))	

   * Specifies the alignment of the column  heading and the subitem text.
   function fmt(n)                      
	return iif(pcount()=0,this.member(4),this.member(4,n))
   	
   * Specifies the width in Pixels 
   function cx(n)                      
   return iif(pcount()=0,this.member(8),this.member(8,n))

   * c is a normal string (Unicode)  or an integer (pointer)
   function pszText(c)
	   if pcount()=0
   		return this.gettext(12,this.p_psztext)
	   endif
  	return this.settext(c,12,this.p_psztext)
      						
   * Size in characters of pszText
   function cchTextMax(n)   					
	return iif(pcount()=0,this.member(16),this.member(16,n))			

   * Index of Subitem
   function iSubItem(n)   					
	return iif(pcount()=0,this.member(20),this.member(20,n))					

   * Cleanup
   function release()
   	if this.p_psztext#0
      	globalfree(this.p_psztext)
      endif
      super::release()
   return

endclass								///////////  End of Class LV_Column

*********************
* LV_ITEM structure
*********************
/*
   The LV_ITEM structure specifies or receives the attributes of a 
   list view item.

   typedef struct _LV_ITEM {  
       UINT   mask; 
       int    iItem; 
       int    iSubItem; 
       UINT   state; 
       UINT   stateMask; 
       LPTSTR  pszText; 
       int    cchTextMax; 
       int    iImage;       // index of the list view item's icon 
       LPARAM lParam;       // 32-bit value to associate with item 
   } LV_ITEM; 
*/ 

class LV_ITEM of Struct_Base(36)

	this.p_psztext=0		// pointer to psztext

   * a set of bit flags that specify   attributes of the struct
	function mask(n)							
	return iif(pcount()=0,this.member(0),this.member(0,n))				

   * Specifies the zero based index of the item
   function iItem(n)                   
	return iif(pcount()=0,this.member(4),this.member(4,n))					

   * Specifies the one based index  of the subitem (0 if item)
   function iSubItem(n)                
	return iif(pcount()=0,this.member(8),this.member(8,n))	

   * Current state 
   function state(n)                   
	return iif(pcount()=0,this.member(12),this.member(12,n))	

   * Specifies the bits of the state member that are valid.
   function statemask(n)                   
	return iif(pcount()=0,this.member(16),this.member(16,n))	

   * c is a normal string (Unicode)  or an integer (pointer)
   function pszText(c)
	   if pcount()=0
   		return this.gettext(20,this.p_psztext)
	   endif
  	return this.settext(c,20,this.p_psztext)

   * Size in characters of pszText
   function cchTextMax(n)   					
	return iif(pcount()=0,this.member(24),this.member(24,n))					

   * index of the list view item's icon 
   function iImage(n);       //    					
	return iif(pcount()=0,this.member(28),this.member(28,n))	

   * 32-bit value to associate with item 
   function lParam(n);       //    					
	return iif(pcount()=0,this.member(32),this.member(32,n))	

   * Cleanup
   function release()
   	if this.p_psztext#0
      	globalfree(this.p_psztext)
      endif
      super::release()
   return

endclass								///////////  End of Class LV_ITEM

******************
* DRAWITEMSTRUCT
*****************

/*
   The DRAWITEMSTRUCT structure provides information the owner window 
   must have to determine how to paint an owner-drawn control or 
   menu item. The owner window of the owner-drawn control or 
   menu item receives a pointer to this structure as the lParam 
   parameter of the WM_DRAWITEM message. 

   typedef struct tagDRAWITEMSTRUCT {  // dis  
       UINT  CtlType; 
       UINT  CtlID; 
       UINT  itemID; 
       UINT  itemAction; 
       UINT  itemState; 
       HWND  hwndItem; 
       HDC   hDC; 
       RECT  rcItem; 
       DWORD itemData; 
   } DRAWITEMSTRUCT; 
*/

class DRAWITEMSTRUCT of Struct_Base(48)

   * Specifies the control type.
	function CtlType(n)							
	return iif(pcount()=0,this.member(0),this.member(0,n))				

   * Specifies the identifier 
   function CtlID(n)                   
	return iif(pcount()=0,this.member(4),this.member(4,n))					

   * Specifies the index of the item
   function itemID(n)                
	return iif(pcount()=0,this.member(8),this.member(8,n))	

   * Specifies the drawing action required.
   function itemAction(n)                   
	return iif(pcount()=0,this.member(12),this.member(12,n))	

   * Specifies the visual state of the item after the current
   * drawing action takes place
   function itemState(n)                   
	return iif(pcount()=0,this.member(16),this.member(16,n))	

   * Identifies the control
   function hwndItem(n)   					
	return iif(pcount()=0,this.member(20),this.member(20,n))					

   * Identifies a device context; this device context must be used
   *when performing drawing operations on the control. 
   function hDC(n);       //    					
	return iif(pcount()=0,this.member(24),this.member(24,n))	

   * Specifies the application-defined 32-bit value 
   function itemData(n);       //    					
	return iif(pcount()=0,this.member(44),this.member(44,n))	

   *Specifies a rectangle that defines the boundaries of the control
   function getrcItem();       //    					
	return substr(this.string,14,8)	

   * Cleanup
   function release()
      super::release()
   return

endclass								///////////  End of Class DrawItemStruct

********************
* MEASUREITEMSTRUCT
********************

/*
   The MEASUREITEMSTRUCT structure informs Windows of the dimensions 
   of an owner-drawn control or menu item. 
   This allows Windows to process user interaction with the control 
   correctly. 

   typedef struct tagMEASUREITEMSTRUCT {   // mis  
       UINT  CtlType;      // type of control 
       UINT  CtlID;        // combo box, list box, or button identifier 
       UINT  itemID;       // menu item, variable-height list box identifier 
       UINT  itemWidth;    // width of menu item, in pixels 
       UINT  itemHeight;   // height of single item in list box menu, in pixels 
       DWORD itemData;     // application-defined 32-bit value 
   } MEASUREITEMSTRUCT; 

*/

class MEASUREITEMSTRUCT of Struct_Base(24)

   * Specifies the control type.
	function CtlType(n)							
	return iif(pcount()=0,this.member(0),this.member(0,n))				

   * Specifies the identifier 
   function CtlID(n)                   
	return iif(pcount()=0,this.member(4),this.member(4,n))					

   * Specifies the index of the item
   function itemID(n)                
	return iif(pcount()=0,this.member(8),this.member(8,n))	

   * Specifies the width, in pixels, of a item.
   function itemWidth(n)                   
	return iif(pcount()=0,this.member(12),this.member(12,n))	

   * Specifies the height, in pixels, of an individual item in a list box 
   * or a menu. Before returning from the message, the owner of the 
   * owner-drawn combo box, list box, or menu item must fill out this member. 
   function itemHeight(n)                   
	return iif(pcount()=0,this.member(16),this.member(16,n))	

   * Specifies the application-defined 32-bit value
   function ItemData(n)   					
	return iif(pcount()=0,this.member(20),this.member(20,n))

   * Cleanup
   function release()
      super::release()
   return
		
endclass								///////////  End of Class MeasureItemStruct

************************
* progresscallbackstruct
************************

/* 
   Used to implement a progressbar with the  
   UrlDownloadToFile API-Function

   Members : 	purl			: pointer to string containing the url to download
               pfname		: pointer to string containing the filenema
               eventhandle	: handle of the event to synchronize the threads
               pstradr		: address of the structure
               icancel		: flag to cancel download 0=no cancel
               wndhnd		: windowhandle of progressbar
*/

class ProgressCallbackStruct of Struct_Base(24)

	this.p_purl=0	
   this.p_pfname=0

   * c is a normal string (Unicode)  or an integer (pointer)
   * it is the url-address from where to download 
   function purl(c)
	   if pcount()=0
   		return this.gettext(0,this.p_purl)
	   endif
  	return this.settext(c,0,this.p_purl)

   * c is a normal string (Unicode)  or an integer (pointer)
   * it is the filename (with path) to write the downloaded file to
   function pfname(c)
	   if pcount()=0
   		return this.gettext(4,this.p_pfname)
	   endif
  	return this.settext(c,4,this.p_pfname)

   * Contains an event-handle to synchronize the download-thread with 
   * the Progress-Bar-Thread. (This is not really multithreading, because
   * the download-thread has to wait for dbase to perform the messagehandling.
   * It is more some kind of serialization)
   function eventhandle(n)   					
	return iif(pcount()=0,this.member(8),this.member(8,n))		

   * Contains the address of this structure
   function pstradr(n)
   return iif(pcount()=0,this.member(12),this.member(12,n))

   * flag to cancel the download 0 if download has to go on 
   function icancel(n)
   return iif(pcount()=0,this.member(16),this.member(16,n))

   * windowhandle of progressbar (used to send the progressmessages)
   function wndhnd(n)
   return iif(pcount()=0,this.member(20),this.member(20,n))

   * Cleanup
   function release()
   	if this.p_purl#0
      	globalfree(this.p_purl)
      endif
      if this.p_pfname # 0
      	globalfree(this.p_pfname)
      endif
      super::release()
   return

endclass								///////////  End of Class ProgressCallbackStruct

*************
* FLDDesc
*************

/* 
  Field descriptor is a structure used by the bde both in creating a 
  and in inquiring about the table structure after it is opened

   Members :	iFldNum		spedifies the field number
               szName		name of the field (all sz are null terminated)
               iFldType		physical or logical bde type (translate mode)
               iSubType		???
               iUnits1		number of characters or digits
               iUnits2		number of decimal places
               iOffset		offset in record buffer ???
               iLen			length of field in bytes (translate mode)
               iNullOffset	offset of Null indicator in record buffer ???
               efldvVchk	type of validity-check
               efldrrights	field level rights
               bCalcField	true if field is a calculated field
               iUnUsed		reserved
*/

Class FLDDesc of Struct_Base(62)
   function iFldNum(n)                   
	return iif(pcount()=0,this.wordmember(0),this.wordmember(0,n))	

   function szname(c)                   
	return iif(pcount()=0,this.Ansistring(2,32),this.ansistring(2,32,c))	

   function iFldType(n)   					
	return iif(pcount()=0,this.wordmember(34),this.wordmember(34,n))

   function isubtype(n)   					
	return iif(pcount()=0,this.wordmember(36),this.wordmember(36,n))
   
   function iunits1(n)   					
	return iif(pcount()=0,this.wordmember(38),this.wordmember(38,n))

   function iunits2(n)   					
	return iif(pcount()=0,this.wordmember(40),this.wordmember(40,n))

   function ioffset(n)   					
	return iif(pcount()=0,this.wordmember(42),this.wordmember(42,n))

   function ilen(n)   					
	return iif(pcount()=0,this.wordmember(44),this.wordmember(44,n))

   function inulloffset(n)   					
	return iif(pcount()=0,this.wordmember(46),this.wordmember(46,n))

   function efldvvchk(n)   					
	return iif(pcount()=0,this.wordmember(48),this.wordmember(48,n))

   function efldrrights(n)   					
	return iif(pcount()=0,this.wordmember(50),this.wordmember(50,n))

   function bcalcfield(n)   					
	return iif(pcount()=0,this.wordmember(52),this.wordmember(52,n))

   * Cleanup
   function release()
      super::release()
   return

endclass					// end of class flddesc

**********
*CURProps
**********

/* 
   the cursor properties structure describes the most commonly used 
   cursor properties

   szName 			Specifies the table name. 
   iFNameSize 		Specifies the size of the buffer needed to retrieve full 
                  table name (including extension and path, if applicable).
   szTableType 	Specifies the driver type. 
   iFields 			Specifies the number of fields in the table. The client must
                  allocate a buffer whose size is: [iFields * sizeof(FLDDesc)] 	
   iRecSize 		Specifies the record size, depending on the xltMODE for the 
                  cursor. If the xltMODE is xltFIELD, iRecSize specifies the 
                  logical record size. In other words, it is the size of the 
                  record if all fields were represented as BDE logical types. 
                  If the xltMODE is xltNONE, iRecSize specifies the physical
                   record size.
   iRecBufSize 	Specifies the physical record size. This is the size of the 
                  record buffer that the client must allocate in order to retrieve
                  the records using DbiGetNextRecord, DbiGetPriorRecord, and other
                  functions. This size can change if DbiSetFieldMap is called.
   iKeySize 		Specifies the key size of the current active index (if any). 
   iIndexes 		Specifies the number of currently open indexes for this cursor.
   iValChecks 		Specifies the number of validity checks existing for this table.
   iRefIntChecks 	Specifies the number of referential integrity constraints 
   iBookMarkSize 	Specifies the size of the bookmark. 
   bBookMarkStable TRUE, if this cursor supports stable bookmarks. 
   eOpenMode 		Specifies the open mode that this cursor was opened with.
   eShareMode		Specifies the share mode that this cursor was opened with:
   bIndexed 		This value is TRUE if there is a current active index for this
                  cursor. 
   iSeqNums 		1	This cursor supports the sequence number concept (Paradox).
                  0	This cursor supports the record number concept (dBASE,FoxPro).
                  < 0 (-1, -2. . .): None (SQL and Access)
   bSoftDeletes 	This value is set to TRUE if this cursor supports soft deletes 
   bDeletedOn 		This value is set to TRUE if the curSOFTDELETEON property is TRUE
   iRefRange 		Not currently used.
   exltMode 		Specifies the value of the translate mode property for cursor.
   iRestrVersion 	Specifies the restructure version number for the table. 
   bUniDirectional This value is set to TRUE if this cursor is unidirectional 
   eprvRights 		Specifies an enumerated value that gives the table-level rights 
                  for the user who opened the table.
   iFmlRights 		Reserved.
   iPasswords 		Specifies the number of auxiliary passwords for this table. 
   iCodePage 		Specifies the code page associated with the table. 
                  If the code page is unknown, the value is 0.
   bProtected 		This value is set to TRUE if the table is protected by password.
   iTblLevel 		Specifies the table level. This value is driver dependent.
   szLangDriver 	Specifies the name of the language driver associated with table.
   bFieldMap 		This value is set to TRUE if a field map is active for  cursor.
   iBlockSize 		Specifies the value of the BLOCKSIZE for the table, in bytes.
   bStrictRefInt 	
   iFilters			Specifies the number of filters currently on the cursor.
   bTempTable 		TRUE, if the cursor is on a temporary table. 
*/

Class CurProps of Struct_Base(427)

* because this structure is readonly, we don' t need the memberfunctions
* but can use directly the get-functions

   function szname() ; return this.getansistring(0,261)
   function ifnamesize() ; return this.getword(261)
   function sztabletype() ; return this.getansistring(263,32)
   function ifields() ; return this.getword(295)
   function irecsize() ; return this.getword(297)
   function irecbufsize() ; return this.getword(299)
   function ikeysize() ; return this.getword(301)
   function iTblLevel() ; return this.getword(351)

   * not complete
endclass				// end of class CURProps

***************
* StringPointer
***************

/* 
   not really a structure, but can be used if an external call 
   uses a pointer to a string as parameter ( dbiopentable )
*/ 

class Stringpointer of struct_base(4)

	this.p_string=0		// pointer to psztext

   function pstring(c)
	   if pcount()=0
   		return this.gettext(0,this.p_string)
	   endif
  	return this.settext(c,0,this.p_string)

   * Cleanup
   function release()
   	if this.p_string#0
      	globalfree(this.p_string)
      endif
      super::release()
   return

endclass 			// end of class Stringpointer

****************
* Handlepointer
****************

/* 
   See StringPointer, but for handles
*/

class HandlePointer of struct_base(4)

   function hnd(n)                      
   return iif(pcount()=0,this.member(0),this.member(0,n))

   * Cleanup
   function release()
      super::release()
   return

endclass				// end of class Handlepointer

*************
* CRTblDesc
*************

/* 
   A structure used in the bde. This structure is used by 
   DbiDoRestructure and DbiCreateTable to either change the 
   fields of a table or to create a new table

   Only the most important members are surfaced here 

   SZTBLNAME			: source table name
   ifldcount			: number of field operators
   pecrFldOp			: pointer to an array of CropType structures
   pFLDDesc				: pointer to array of flddesc structures
   sztbltype			: table driver type
   ioptParams			: number of optional parameters
   pfldOptParams		: pointer to array of optional parameters
   pOPTData				: specifies values of optional parameters	
*/

class CRTblDesc of Struct_Base(682)

	this.p_crop=0	
   this.p_flddesc=0

   /* 
      We can not use the base-method settext, because it will try 
      to transform our structure - string from Unicode to Ansi, 
      which is not desired in this case, because it is a 
      structure and not a string. Therefor we use a slightly 
      different method (unique to this structure)
   */ 
   function getxtext(nIndex,pointer)
   	* no implementation yet
   return

   function setxtext
	  	parameters c,n,h		// pass pointer as reference so it can be changed                  
   	local apierror,cResult,hprocess
	   cResult =""
		if h#0					// release the old memory-block so that it can be 
      	globalfree(h)		// reused later (and to avoid memory-leaks)
      endif
  	   h=globalalloc(GMEM_FIXED,len(c)*2+1)		// get the address
      if h=0
        	Msgbox("Memory allocation failed","Fatal Error",0)
      else
        	hprocess=getcurrentprocess()
        * write the string to the reserved memory-location but dont transform
			apierror=writeprocessmemoryp(hprocess,h,c+chr(0),len(c)*2+1,0)
			if not apierror
  	        	Msgbox("Pointer creation failed","Fatal Error",0)
         else 
           	cresult=str(h)
            this.setlong(n,h)
         endif
      endif
	return cresult	 // either empty string (on error) or pointer to the string
	
   * c is the pcroptype structure
   function pcroptype(c)
	   if pcount()=0
   		return this.getxtext(624,this.p_crop)
	   endif
  	return this.setxtext(c,624,this.p_crop)

	* holds the flddesc array of structures
   function pflddesc(c)
	   if pcount()=0
   		return this.getxtext(628,this.p_flddesc)
	   endif
  	return this.setxtext(c,628,this.p_flddesc)

   function ioptparams(n)   					
	return iif(pcount()=0,this.wordmember(672),this.wordmember(672,n))

   function pfldoptparams(c)
	   if pcount()=0
   		return this.getxtext(674,this.p_flddesc)
	   endif
  	return this.setxtext(c,674,this.p_flddesc)

   function poptdata(c)
	   if pcount()=0
   		return this.gettext(678,this.p_crop)
	   endif
  	return this.settext(c,678,this.p_crop)

  
   function sztblname(c)                   
	return iif(pcount()=0,this.Ansistring(0,261),this.ansistring(0,261,c))	

   function iFldcount(n)   					
	return iif(pcount()=0,this.wordmember(622),this.wordmember(622,n))

   function sztbltype(c)
   return iif(pcount()=0,this.ansistring(261,32),this.ansistring(261,32,c))

   * Cleanup
   function release()
   	if this.p_crop#0
      	globalfree(this.p_crop)
      endif
      if this.p_flddesc # 0
      	globalfree(this.p_flddesc)
      endif
      super::release()
   return

endclass								///////////  End of Class CRTbldesc


