/* File 		: urld.cc
Autor    : Marc Van den Berghen

Make a small form to download a file from the internet.
This uses the UrlDownloadToFile API-Function.
It implements a progress-bar and a Cancel-Button to show the progress
in lengthy downloads.

Dependencies : api_structs.cc
progresscallback.dll

Example : local f
f= new dwnldcnt("www.dbase.com",".\test.htm")
f.readmodal()
result = f.errorcode
f.release()

where the first parameter is the url from where to download
and the second parameter is the filename (with path) where to
save to.

result =0 --> download succesfull
result =1 --> download cancelled by user
result =2 --> no connection to the internet
result =3 --> download failed.

History : Version 1.0 		06-Sep-03		Marc Van den Berghen		
first Beta
Version 1.1		09-Sep-03		Marc Van den Berghen
added errorcode to check for psiible failures
09-Sep-03		Rich (Autotraker)
Thanks to Rich the cache is set off temporarily
09-Sep-03 		Marc Van den Berghen
Bugfix - when not connected to Internet

12-Jan-04      Bernd Hohenester
added DeleteUrlCacheEntry() after a hint from Marc
to insure that the file ist always downloaded from server
added InternetAutoDial and InternetAutoDialHangup to check
the connection

13-Jan-04 added custom-property autodial
autodial=true then the class will try to establish
an internet connection
autodial=false then the class will return if not connected

*/

class dwnldcnt(curl,cfname) of form custom
  
  set procedure to api_structs.cc additive	
  
  this.metric		:= 6
  this.onopen		:= class::initialize
  this.onclose	:= class::cleanup
  this.left	  	:= 20
  this.top		  	:= 30
  this.width 		:= 350
  this.height 	:= 205
  this.mdi			:= false
  this.text		:= "Download from : "+curl
  
  
  * Custom properties
  
  this.url			= curl			
  this.fname		= cfname
  this.sync		= new progresscallbackstruct()
  this.evnthnd	= 0
  this.threadhnd	= 0
  this.errorcode	= 3
  this.percent	= 0
  this.autodial	= false
  
  * Paintbox used to synchronize the dbase with the download-thread
  
  this.PNTBX1 = new PAINTBOX(this)
  with (this.pntbx1)
    height 		:= 0
    left 			:= 0
    top 			:= -200
    width 		:= 0
    onleftmouseup := class::lmu 		// notification of any progress
    onleftmousedown :=class::lmd     // notification of thread-termination
  endwith
  
  this.PB1 = new PUSHBUTTON(this)
  with (this.pb1)
    height 		:= 24
    left 			:= 221
    top 			:= 173
    width 		:= 114
    text 			:= "Cancel download"
    onclick		:= class::cancel
  endwith
  
  this.PB2 = new PUSHBUTTON(this)
  with (this.pb2)
    height 		:= 24
    left 			:= 221
    top 			:= 173
    width 		:= 114
    text 			:= "Close Form"
    visible		:= false
    onclick		:= { ; this.parent.close(this.parent.errorcode)}
  endwith
  
  this.PROGRESS1 = new PROGRESS(this)
  with (this.PROGRESS1)
    height 		:= 22
    left 			:= 18
    top 			:= 41
    width 		:= 308
    value 		:= 0
    rangeMin 	:= 0
    rangeMax 	:= 100
  endwith
  
  * Indicates the file to download to
  
  this.TLABEL1 = new TEXTLABEL(this)
  with (this.TLABEL1)
    height 		:= 22
    left 			:= 18
    top   		:= 74
    width 		:= 308
    text 			:= "Download to : "+cfname
  endwith
  
  * Percent of download
  
  this.TLABEL2 = new TEXTLABEL(this)
  with (this.TLABEL2)
    height 		:= 22
    left 			:= 18
    top 			:= 107
    width 		:= 328
    text 			:= ""
  endwith
  
  this.check1=new checkbox(this)
  with (this.check1)
    height		:= 22
    left			:= 18
    top			:= 140
    width			:= 300
    text			:= "Close form when download completed"
    value			:= false
  endwith
  
  function initialize()
    local thr_adr
    extern chandle init(cptr) progresscallback
    extern chandle CreateEventA(cptr,clogical,clogical,cptr) kernel32
    extern chandle CreateThread(cptr,clong,cptr,cptr,clong,cptr) kernel32
    extern clogical SetEvent(chandle) kernel32
    extern clogical CloseHandle(chandle) kernel32
    extern clong InternetGetConnectedState(cptr,clong) wininet
    extern clogical TerminateThread(chandle,clong) kernel32
    extern clogical InternetGetConnectedState(cLong,cLong) "wininet.dll"
    
    
    // added by Bernd Hohenester
    extern clogical DeleteUrlCacheEntry(cstring) wininet
    extern clogical InternetAutodial(clong,chandle) wininet
    extern clogical InternetAutodialHangup(clong) wininet
    
    if not InternetGetConnectedState(0,0)
      if this.autodial
        if not InternetAutodial(0,0)        // changed by Bernd Hohenester
          this.tlabel2.text="No connection to the Internet"
          this.check1.visible=false
          this.pb1.visible=false
          this.pb2.visible=true
          this.errorcode=2
          return
        endif
      else
        this.tlabel2.text="No connection to the Internet"
        this.check1.visible=false
        this.pb1.visible=false
        this.pb2.visible=true
        this.errorcode=2
        return
      endif
    endif
    DeleteUrlCacheEntry(this.url)    // added by Bernd Hohenester
    this.evnthnd= CreateeventA(null,false,false,null)
    this.sync.eventhandle(this.evnthnd)
    this.sync.icancel(0)
    this.sync.pstradr(0)
    this.sync.wndhnd(this.pntbx1.hwnd)
    this.sync.purl(this.url)
    this.sync.pfname( this.fname)
    thr_adr=init(this.sync)
    this.threadhnd = createthread(null,0,thr_adr,this.sync.pstradr(),0,'  ')
  return
  
  function cleanup()
    // added by Bernd Hohenester
    if this.autodial
      if not InternetAutodialHangUp(0)
        msgbox("Couldn't terminate connection to the internet!","",48)
      endif
    endif
    this.sync.icancel(1)
    setevent(this.sync.eventhandle())
    closehandle(this.evnthnd)
    closehandle(this.threadhnd)
    this.sync.release()
  return
  
  function lmu(f, c, r)
    local dmax,p
    c = iif(c<0,65536+c,c)
    r = iif(r<0,65536+r,r)
    p=this.parent
    dmax=int(r*65536+c)
    dmax=iif(dmax=0,1,dmax)
    p.percent= int(f/dmax*100+.5)
    p.progress1.value=p.percent
    p.tlabel2.text= " Saved "+f+" bytes of "+dmax+" ("+p.percent+"%)"
    setevent(p.sync.eventhandle())
  return
  
  function lmd()
    local p
    p=this.parent
    if p.percent = 100
      p.tlabel2.text="Download complete (100%)"
      p.errorcode=0
    else
      p.tlabel2.text="Download failed"
      p.errorcode=3
    endif
    if p.check1.value
      p.close()
    else
      p.pb1.visible=false
      p.pb2.visible=true
    endif
  return
  
  function cancel()
    this.parent.sync.icancel(1)
    setevent(this.parent.sync.eventhandle())
    this.parent.errorcode=1
    this.parent.close()
  return
  
endclass


